@@ -4,7 +4,7 @@ import subprocess
 import sys
 import asyncio
 import time
-
+import atexit
 import requests
 import tempfile
 import logging
@@ -14,7 +14,7 @@ import socket
 import websockets
 from websockets.exceptions import InvalidStatusCode, ConnectionClosed, ConnectionClosedOK
 from logging.handlers import TimedRotatingFileHandler
-from tickerdax.constants import Envs, KeyTypes, NAME, URL
+from tickerdax.constants import Envs, KeyTypes, NAME, URL, GLOBAL_INSTANCE_NAME
 from docker.errors import DockerException
 from datetime import datetime, timedelta, timezone
 from tickerdax import formatting
@@ -41,6 +41,8 @@ class TickerDax:
             websocket_api_prefix = 'dev-ws'
 
         # general configuration
+        # Todo supported timeframe needs to be checked per route
+        #  https://github.com/TickerDax/tickerdax-client/issues/2
         self.supported_timeframes = ['1h']
         self.rest_values = []
         self.cached_values = []
@@ -73,6 +75,9 @@ class TickerDax:
         self._docker_client = None
         self.redis_client = None
 
+        # module level dictionary for caching during runtime
+        self._runtime_cache = {}
+
         self._cache_folder = os.environ.get(Envs.CACHE_ROOT.value, os.path.join(tempfile.gettempdir(), f'{NAME}_cache'))
         self._logger = logging.getLogger(self.__class__.__name__)
         self._logger.setLevel(level=logging.DEBUG)
@@ -83,6 +88,9 @@ class TickerDax:
         if connect:
             self._start_redis_server()
 
+        # set this instance so it can be accessed globally
+        globals()[GLOBAL_INSTANCE_NAME] = self
+
     @staticmethod
     def _get_cache_keys(route, symbols, timestamps, timeframe):
         """
@@ -125,6 +133,20 @@ class TickerDax:
             sock.bind(('', 0))
             return sock.getsockname()[1]
 
+    def _get_from_runtime_cache(self, route, symbols, since, till, timeframe):
+        """
+        Get data from the runtime cache.
+
+        :param str route: The data route.
+        :param list[str] symbols: A list of symbols.
+        :param datetime since: The UTC start time.
+        :param datetime till: The UTC end time.
+        :param str timeframe: The time interval.
+        :returns: The sorted result.
+        :rtype list
+        """
+        return self._runtime_cache.get(f'{route}{symbols}{since}{till}{timeframe}')
+
     def _get_from_cache(self, keys):
         """
         Get the data from the cache that already exists, and which REST requests still need to be made.
@@ -167,6 +189,19 @@ class TickerDax:
         # return the needed rest requests
         return rest_requests
 
+    def _update_runtime_cache(self, route, symbols, since, till, timeframe, result):
+        """
+        Set the result data in the runtime cache.
+
+        :param str route: The data route.
+        :param list[str] symbols: A list of symbols.
+        :param datetime since: The UTC start time.
+        :param datetime till: The UTC end time.
+        :param str timeframe: The time interval.
+        :param list[dict] result: The resulting data from the get_route request.
+        """
+        self._runtime_cache[f'{route}{symbols}{since}{till}{timeframe}'] = result
+
     def _setup_logger(self):
         """
         Sets up the logger.
@@ -542,6 +577,17 @@ class TickerDax:
             )
         return result
 
+    @staticmethod
+    @atexit.register
+    def on_shutdown():
+        """
+        This runs when python shuts down.
+        """
+        self = globals()[GLOBAL_INSTANCE_NAME]
+        self._logger.info('Saving data to disk..')
+        self.redis_client.bgsave()
+        self._logger.info('Save complete!')
+
     def validate_api_key(self,  key_type):
         """
         Validate whether the key of the given type exists and show and error message.
@@ -583,6 +629,16 @@ class TickerDax:
         :returns: The sorted result.
         :rtype list
         """
+        # truncate times to the timeframe
+        since = formatting.get_unix_time(since, timeframe)
+        till = formatting.get_unix_time(till, timeframe)
+
+        # first check the runtime cache and see this result was already cached
+        cached_result = self._get_from_runtime_cache(route, symbols, since, till, timeframe)
+        if cached_result:
+            return cached_result
+
+        # clear out which values are being tracked as cached, rest, or missing values
         self.cached_values.clear()
         self.rest_values.clear()
         self.missing_values.clear()
@@ -601,8 +657,12 @@ class TickerDax:
         else:
             self.rest_values = self._request(route, outstanding_rest_requests, timeframe)
 
-        result = self._update_cache(route, keys)
-        return sorted(result, key=lambda i: i['timestamp'])
+        result = sorted(self._update_cache(route, keys), key=lambda i: i['timestamp'])
+
+        # update the runtime cache so that data is not re-requested within the same runtime
+        self._update_runtime_cache(route, symbols, since, till, timeframe, result)
+
+        return result
 
     def report_error(self, message):
         """
